/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.ListDialog;

import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;

/**
 * <p>ChooseClassificationWizard class.</p>
 *
 * @author n.hoffmann
 * @created 19.03.2009
 * @version 1.0
 */
public class ChooseFromMultipleTaxonNodesDialog extends ListDialog {

	public static TaxonNode choose(Set<TaxonNode> taxonNodes){
		ChooseFromMultipleTaxonNodesDialog dialog = new ChooseFromMultipleTaxonNodesDialog(EditorUtil.getShell());
		dialog.setInput(taxonNodes);
		int result = dialog.open();
		
		if(result == IStatus.OK){
			Classification selectedClassification = (Classification) dialog.getResult()[0];
			
			for(TaxonNode taxonNode : taxonNodes){
				if(taxonNode.getClassification().equals(selectedClassification)){
					return taxonNode;
				}
			}
		}
		return null;
	}
	
	public ChooseFromMultipleTaxonNodesDialog(Shell parent) {
		super(parent);
		setTitle(Messages.ChooseFromMultipleTaxonNodesDialog_CHOOSE_CLASSIFICATION);
		setMessage(Messages.ChooseFromMultipleTaxonNodesDialog_CHOOSE_CLASSIFICATION_MESSAGE);
		setContentProvider(new ClassificationContentProvider());
		setLabelProvider(new ClassificationLabelProvider());
	}
	
	private class ClassificationLabelProvider extends LabelProvider{
		@Override
		public String getText(Object element) {
			return super.getText(((Classification) element).getTitleCache());
		}
	}
	
	private class ClassificationContentProvider implements IStructuredContentProvider{

		@Override
		public Object[] getElements(Object inputElement) {
			Set<TaxonNode> taxonNodes = (Set<TaxonNode>) inputElement;
				
				List<Classification> classifications = new ArrayList<Classification>(); 
				
				for(TaxonNode node : taxonNodes){
					classifications.add(node.getClassification());
				}
				
				return classifications.toArray();
		}
		
		@Override
		public void dispose() {
			
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			
		}		
	}
}
