/**
* Copyright (C) 2011 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IMemento;

import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.ContextListenerAdapter;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.UseObjectStore;

/**
 * The context listener will call this class when a new context is started.
 * It will reset the UseObjectStore to guarantee the static
 * content of the store will be refreshed each time we load a new context.
 * @author a.theys
 * @author a.mueller
 * @created mar 13, 2012
 */
public class UseObjectManager extends ContextListenerAdapter{

    private static final String RESET_USAGE_DATA = Messages.UseObjectManager_RESET_DATA;


    public UseObjectManager() {
		CdmStore.getContextManager().addContextListener(this);
	}


	@Override
	public void contextRefresh(IProgressMonitor monitor) {
		monitor.subTask(RESET_USAGE_DATA);
		MessagingUtils.warn(getClass(), RESET_USAGE_DATA);
		UseObjectStore.reset();
	}


	@Override
	public void contextStart(IMemento memento, IProgressMonitor monitor) {
//		monitor.subTask("Starting the Uses View");
//		MessagingUtils.warn(getClass(), "Starting Uses View warn");
		UseObjectStore.reset();
	}


}
