/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.group.authority;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.ui.group.grantedauthority.CdmAuthorityCompositeViewer;

/**
 * Editor responsible for editing authorities related to a particular {@link Group} entity.
 *
 * @author cmathew
 * @created Mar 28, 2013
 *
 */
public class CdmAuthorityEditor extends EditorPart implements IConversationEnabled, IDirtyMarkable {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.group.authority"; //$NON-NLS-1$

	private boolean dirty;
	private ConversationHolder conversation;
	private CdmAuthorityCompositeViewer viewer;

	public CdmAuthorityEditor() {

	}

	/**
	 * Create contents of the editor part.
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new FillLayout(SWT.HORIZONTAL));

		this.viewer = new CdmAuthorityCompositeViewer(container, this,((CdmAuthorityEditorInput) getEditorInput()).getGroup());

		conversation = ((CdmAuthorityEditorInput) getEditorInput()).getConversationHolder();
		setPartName(getEditorInput().getName());

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
	    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell().setFocus();
	    ((CdmAuthorityEditorInput) getEditorInput()).bind();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		try {
			monitor.beginTask(Messages.CdmAuthorityEditor_SAVING_AUTHORITY_EDITOR, 1);
			getConversationHolder().commit(true);
			((CdmAuthorityEditorInput) getEditorInput()).merge();
			dirty = false;
			firePropertyChange(PROP_DIRTY);
			monitor.worked(1);
		} finally {
			monitor.done();
			viewer.save();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {
		// Do the Save As operation
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return dirty;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap changeEvents) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#changed(java.lang.Object)
	 */
	@Override
	public void changed(Object element) {
		//FIXME : should be optimised to split changes into adding / removing authorities
		//        vs updating authorites
		dirty = viewer.isDirty();
		firePropertyChange(PROP_DIRTY);
		// if the change has happened on the group then refresh the table
		if(element instanceof Group) {

			//FIXME: activating (setting focus) on the editor happens on every change
			//       This should be changed to only when the drop is successful
			getSite().getPage().activate(this);
		}

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#forceDirty()
	 */
	@Override
	public void forceDirty() {
	    changed(null);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.eclipse.ui.forms.editor.FormEditor#dispose()
	 */
	@Override
	public void dispose() {
		conversation.unregisterForDataStoreChanges(this);
		conversation.close();
		((CdmAuthorityEditorInput) getEditorInput()).getCdmEntitySession().dispose();
		super.dispose();
	}
}
