package eu.etaxonomy.taxeditor.editor.handler;


import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;



/**
 * Tests types of specimens to add items to the context menu.
 *
 * @author pplitzner
 * @author BenStoever
 */
public class SpecimenPropertyTester extends PropertyTester {
    private static final String SEQUENCE = "isSequence"; //$NON-NLS-1$
    private static final String SINGLE_READ = "isSingleRead"; //$NON-NLS-1$
    private static final String IS_SINGLEREAD_REUSABLE_HERE = "isSingleReadReusableHere"; //$NON-NLS-1$
    private static final String IS_SINGLEREAD_REUSED = "isSingleReadReused"; //$NON-NLS-1$


    public SpecimenPropertyTester() {}

    /** {@inheritDoc} */
    @Override
    public boolean test(Object receiver, String property, Object[] args, Object expectedValue) {
        if (receiver instanceof IStructuredSelection) {
            IStructuredSelection selection = (IStructuredSelection) receiver;
            TreeNode treeNodeOfSelection = EditorUtil.getTreeNodeOfSelection(selection);
            if (treeNodeOfSelection!=null) {
                if (SEQUENCE.equals(property)) {
                    return isSequence(treeNodeOfSelection.getValue());
                }
                else if (SINGLE_READ.equals(property)) {
                	return isSingleReadAlignment(treeNodeOfSelection.getValue());
                }
                else if (IS_SINGLEREAD_REUSABLE_HERE.equals(property)) {
                    return isSingleReadReusableHere(treeNodeOfSelection.getValue());
                }
                else if (IS_SINGLEREAD_REUSED.equals(property)) {
                    return isSingleReadReused(treeNodeOfSelection.getValue());
                }
            }
        }
        return false;
    }

    private boolean isSingleReadReused(Object value) {
        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        final IEditorPart activeEditor = window.getActivePage().getActiveEditor();
        if(value instanceof SingleRead){
            if(activeEditor instanceof DerivateView
                    && ((DerivateView) activeEditor).getMultiLinkSingleReads().contains(value)){
                return true;
            }
        }
        return false;
    }

    private boolean isSingleReadReusableHere(Object value) {
        TreeNode clipboardNode = EditorUtil.getTreeNodeOfSelection(LocalSelectionTransfer.getTransfer().getSelection());
        if(value instanceof Sequence && clipboardNode!=null && clipboardNode.getValue() instanceof SingleRead
                && !((Sequence) value).getSingleReads().contains(clipboardNode.getValue())){
            return true;
        }
        return false;
    }

    private boolean isSequence(Object object) {
    	return (object instanceof Sequence);
	}

	private boolean isSingleReadAlignment(Object object) {
    	return (object instanceof SingleRead);
    }
}
