/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous;

import org.apache.commons.lang.StringUtils;
import org.eclipse.draw2d.IFigure;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.zest.core.viewers.IEntityStyleProvider;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.KeyStatement;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.PolytomousKeyRelationship;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * 
 * @author n.hoffmann
 * @created Mar 30, 2011
 * @version 1.0
 */
class PolytomousKeyLabelProvider extends LabelProvider implements
		IEntityStyleProvider {

	public static final String LEAF_BUT_NO_TAXON = Messages.PolytomousKeyLabelProvider_LEAF_BUT_NO_TAXON;
	public static final String NO_NODE_NUMBER = Messages.PolytomousKeyLabelProvider_NO_NODE_NUMBER_SET;
	public static final String EMPTY = ""; //$NON-NLS-1$

	@Override
	public String getText(Object element) {
		if (element instanceof PolytomousKey) {
			return ((PolytomousKey) element).getTitleCache();
		} else if (element instanceof PolytomousKeyNode) {
			PolytomousKeyNode keyNode = (PolytomousKeyNode) element;

			if (keyNode.isLeaf()) {
				if (keyNode.getTaxon() != null) {
					return keyNode.getTaxon().getName().getTitleCache();
				}
				return LEAF_BUT_NO_TAXON;
			}

			if (keyNode.getNodeNumber() != null) {
				String statementLabel = ""; //$NON-NLS-1$
				String nodeNumber = keyNode.getNodeNumber().toString();

				KeyStatement question = keyNode.getQuestion();

				if (question != null) {
					statementLabel += " " //$NON-NLS-1$
							+ question.getLabelText(CdmStore
									.getDefaultLanguage());
				}

				return String.format("%s. %s", nodeNumber, statementLabel); //$NON-NLS-1$
			} else {
				return NO_NODE_NUMBER;
			}
		} else if (element instanceof PolytomousKeyRelationship) {
			PolytomousKeyRelationship relationship = (PolytomousKeyRelationship) element;
			if (relationship.getSource() instanceof PolytomousKeyNode) {
				PolytomousKeyNode destinationNode = (PolytomousKeyNode) relationship
						.getDestination();

				KeyStatement statement = destinationNode.getStatement();

				if (statement != null) {
					Language language = CdmStore.getDefaultLanguage();
					String statementLabel = statement.getLabelText(language);

					return StringUtils.isEmpty(statementLabel) ? EMPTY
							: statementLabel;
				}
			}
		}
		return EMPTY;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#fisheyeNode(java.lang
	 * .Object)
	 */
	@Override
	public boolean fisheyeNode(Object arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getBackgroundColour
	 * (java.lang.Object)
	 */
	@Override
	public Color getBackgroundColour(Object arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getBorderColor(java
	 * .lang.Object)
	 */
	@Override
	public Color getBorderColor(Object arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getBorderHighlightColor
	 * (java.lang.Object)
	 */
	@Override
	public Color getBorderHighlightColor(Object arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getBorderWidth(java
	 * .lang.Object)
	 */
	@Override
	public int getBorderWidth(Object arg0) {
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getForegroundColour
	 * (java.lang.Object)
	 */
	@Override
	public Color getForegroundColour(Object arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getNodeHighlightColor
	 * (java.lang.Object)
	 */
	@Override
	public Color getNodeHighlightColor(Object arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.zest.core.viewers.IEntityStyleProvider#getTooltip(java.lang
	 * .Object)
	 */
	@Override
	public IFigure getTooltip(Object arg0) {
		return null;
	}
}
