/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.key.polytomous.handler;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.editor.key.KeyEditor;
import eu.etaxonomy.taxeditor.editor.key.polytomous.IPolytomousKeyEditorPage;
import eu.etaxonomy.taxeditor.editor.key.polytomous.PolytomousKeyEditorLabels;
import eu.etaxonomy.taxeditor.editor.key.polytomous.operation.RemotingCreatePolytomousKeyNodeOperation;

/**
 * @author cmathew
 * @date 21 Jul 2015
 *
 */
public class RemotingCreateSiblingPolytomousKeyNodeHandler extends AbstractPolytomousKeyNodeHandler {

    private PolytomousKeyNode parentNode;


    public RemotingCreateSiblingPolytomousKeyNodeHandler() {
        super(PolytomousKeyEditorLabels.CREATE_SIBLING_POLYTOMOUS_KEY_NODE_LABEL);
    }
    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#allowOperations(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public IStatus allowOperations(ExecutionEvent event) {
        IEditorPart editor = HandlerUtil.getActiveEditor(event);
        IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getCurrentSelection(event);
        editorPage = (IPolytomousKeyEditorPage) ((KeyEditor) editor)
                .getActiveEditor();
        if (selection.getFirstElement() instanceof PolytomousKeyNode) {
            parentNode = (PolytomousKeyNode) selection.getFirstElement();
        } else {
            return new Status(IStatus.ERROR,
                    "unknown", //$NON-NLS-1$
                    PolytomousKeyEditorLabels.NO_KEY_NODE_FOR_INSERT_NODE_SELECTED_MESSAGE);
        }

        return Status.OK_STATUS;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#prepareOperation(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public AbstractOperation prepareOperation(ExecutionEvent event) {
        return new RemotingCreatePolytomousKeyNodeOperation(event.getTrigger(), false, parentNode.getParent(), editorPage);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#onComplete()
     */
    @Override
    public void onComplete() {
        // TODO Auto-generated method stub

    }

}
