/**
 *
 */
package eu.etaxonomy.taxeditor.editor.key.polytomous.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.application.ICdmRepository;
import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IPolytomousKeyNodeService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 *
 */
public class DeleteNodeOperation extends AbstractPostOperation<PolytomousKeyNode> {

	private final PolytomousKeyNode parent;
	private final PolytomousKeyNode node;
	private final boolean deleteChildren;

	public DeleteNodeOperation(String label, IUndoContext undoContext,
			PolytomousKeyNode node, IPostOperationEnabled postOperationEnabled, boolean deleteChildren) {
	    super(label, undoContext, node, postOperationEnabled);
	    HibernateProxyHelper.deproxy(node, PolytomousKeyNode.class);
		this.node = node;
		this.parent = node.getParent();
		this.deleteChildren = deleteChildren;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		//parent.removeChild(node);
		ICdmRepository controller;

		controller = CdmStore.getCurrentApplicationConfiguration();

		IPolytomousKeyNodeService service = controller.getPolytomousKeyNodeService();
		DeleteResult result;

		if (node.getChildren().size()>0){
			if(deleteChildren) {
				result = service.delete(node.getUuid(), false);
			} else{
				result = service.delete(node.getUuid(), true);
			}
		} else{
			result = service.delete(node.getUuid(), true);
		}

		if (!result.isOk() || result.getExceptions().size() > 0){
			Exception t = new Exception();
			if (result.getExceptions().size() >1){
				for (Exception e:result.getExceptions()){
					t.addSuppressed(e);
				}
			}else {
				t = result.getExceptions().iterator().next();
			}
			MessagingUtils.errorDialog(Messages.DeleteNodeOperation_DELETE_FAILED, getClass(),null, TaxeditorBulkeditorPlugin.PLUGIN_ID, t, true);
		}
		return postExecute(null);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

}
