/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.container;

import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * <p>ConceptContainer class.</p>
 *
 * @author p.ciardelli
 * @created 26.01.2009
 * @version 1.0
 */
public class ConceptContainer extends AbstractGroupedContainer<Taxon> {
	
	private final TaxonRelationshipType relationshipType;
	
		
	private ConceptContainer(TaxonNameEditor editor, AbstractGroup group,
			Taxon relatedTaxon, TaxonRelationshipType relationshipType, 
			boolean acceptedTaxonIsFromTaxon) {
		super(relatedTaxon);

		// FIXME the acceptedTaxonIsFromTaxon is never used and 
		// I can't remember what is was used for in the first place
		
		this.relationshipType = relationshipType;
		showRelationshipType();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	/** {@inheritDoc} */
	@Override
	protected void initializeComposite() {
		
		setIsDraggable(true);
		setFont(getViewerFont());
		
		showSec();
				
		initTextViewer();
	}
	
	@Override
	protected void updateIcon() {
		setIcon(CONCEPT_ICON);
	}
	
	private void showRelationshipType() {
		setNonEditableInfo(relationshipType.getLabel(), true);
	}

	private void showSec() {
		if (getTaxonBase() == null) {
			return;
		}
		
		if (getTaxonBase().getSec() == null) {
			setNonEditableInfo("sec. ???", false); //$NON-NLS-1$
		} else {
			setNonEditableInfo("sec. " + getTaxonBase().getSec().getTitleCache(), false); //$NON-NLS-1$
		}
	}

	/**
	 * <p>getNewInstance</p>
	 *
	 * @param editor a {@link eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor} object.
	 * @param group a {@link eu.etaxonomy.taxeditor.editor.name.container.AbstractGroup} object.
	 * @param relationship a {@link eu.etaxonomy.cdm.model.taxon.TaxonRelationship} object.
	 * @return a {@link eu.etaxonomy.taxeditor.editor.name.container.ConceptContainer} object.
	 */
	public static ConceptContainer getNewInstance(TaxonNameEditor editor, AbstractGroup group, TaxonRelationship relationship){

		TaxonRelationshipType relationshipType = relationship.getType();
		
		if (relationship.getToTaxon().equals(editor.getTaxon())) {
			return new ConceptContainer (editor, group, 
					relationship.getFromTaxon(), relationshipType, false);
		} else {
			return new ConceptContainer (editor, group, 
					relationship.getToTaxon(), relationshipType, true);
		} 
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#getViewerFont()
	 */
	/** {@inheritDoc} */
	@Override
	protected Font getViewerFont() {
		return EditorUtil.getFont(Resources.CONCEPT_FONT);
	}

	/**
	 * <p>getRelatedTaxon</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 */
	public Taxon getRelatedTaxon() {
		return getTaxonBase();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.container.AbstractGroupedContainer#showAnnotations()
	 */
	@Override
	public void showAnnotations() {
		if(getData().getSec() == null){
			getNameViewer().addAnnotation(
					new EditorAnnotation(0, Messages.ConceptContainer_SEC_REQUIRED));
		}
		super.showAnnotations();
	}
	
	/** {@inheritDoc} */
	@Override
	protected void updateNonEditableInfo() {
		showSec();
		showRelationshipType();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer#updateIndent()
	 */
	@Override
	protected void updateIndent() {
		setIndent(CONCEPT_INDENT);
	}
}






