package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.exception.HomotypicalGroupChangeException;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class MoveSynonymToAnotherAcceptedTaxonOperation extends
		AbstractPostTaxonOperation {

	/**
	 * The synonym to be moved.
	 */
	private final Synonym synonym;

	private final Taxon oldTaxon;


	/**
	 * <p>Constructor for MoveSynonymToAnotherAcceptedTaxon.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param synonym a {@link eu.etaxonomy.cdm.model.taxon.Synonym} object.

	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public MoveSynonymToAnotherAcceptedTaxonOperation(String label, IUndoContext undoContext,
			UUID oldTaxonUUID, UUID synonymUUID, Taxon taxon, IPostOperationEnabled postOperationEnabled) {

		super(label, undoContext, taxon, postOperationEnabled);
		List<String> propertyPaths =  new ArrayList<String>();
		propertyPaths.add("synonymRelations"); //$NON-NLS-1$
		this.element = (Taxon) CdmStore.getService(ITaxonService.class).load(taxon.getUuid(),propertyPaths);
		this.oldTaxon = (Taxon) CdmStore.getService(ITaxonService.class).load(oldTaxonUUID);
		this.synonym = (Synonym) CdmStore.getService(ITaxonService.class).load(synonymUUID);
		if(synonym == null){
			throw new IllegalArgumentException(
					"A null synonym was provided."); //$NON-NLS-1$
		}
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

    	// Get synonym name
		TaxonNameBase<?, ?> synonymName = synonym.getName();
		monitor.worked(20);

		// Switch groups

		monitor.worked(40);

		((TaxonNameEditor)EditorUtil.getActiveMultiPageTaxonEditor().getActiveEditor()).getConversationHolder().commit();
		UpdateResult result;

		try {
			result = CdmStore.getService(ITaxonService.class).moveSynonymToAnotherTaxon(synonym,
			        this.element.getUuid(),
			        true,
			        synonym.getType(),
			        null,
			        null,
			        true);
		} catch (HomotypicalGroupChangeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		((TaxonNameEditor)EditorUtil.getActiveMultiPageTaxonEditor().getActiveEditor()).getConversationHolder().commit();

		// Redraw editor if it exists
		return postExecute(synonym);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

}
