/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.view.checklist.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.CreateDescriptionElementOperation;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>CreateDescriptionElementHandler class.</p>
 *
 * @author n.hoffmann
 * @created 16.04.2009
 * @version 1.0
 */
public class CreateDescriptionElementHandler extends AbstractHandler {

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.
	 * ExecutionEvent)
	 */
    /** {@inheritDoc} */
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        IWorkbenchPart part = HandlerUtil.getActivePart(event);
        IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part
                : null;


        DescriptionBase<?> description = null;

        ISelection selection = HandlerUtil.getCurrentSelection(event);
        Object selectedElement = null;
        if (selection instanceof ITreeSelection) {
            TreePath[] paths = ((ITreeSelection) selection).getPaths();
            selectedElement = paths[0].getFirstSegment();
        }
        else if (selection instanceof IStructuredSelection) {
            selectedElement = ((IStructuredSelection) selection).getFirstElement();
        }
        if (selectedElement instanceof DescriptionBase<?>) {
            description = (DescriptionBase<?>) selectedElement;
        }

        if (description != null) {
            AbstractPostOperation operation = null;
            try {
                // TODO use undo context specific to editor
                operation = operationCreationInstance(event.getCommand().getName(), event, description, postOperationEnabled);
                AbstractUtility.executeOperation(operation);
            } catch (NotDefinedException e) {
            	MessagingUtils.warn(getClass(), "Command name not set"); //$NON-NLS-1$
            }
        } else {
            MessagingUtils.error(getClass(), new IllegalArgumentException("Could not determine the taxon description")); //$NON-NLS-1$
            return null;
        }
        return null;

    }

    /**
	 * Added to make the Class more generic and limit the amount of code changes required
	 * @param label
	 * @param event
	 * @param taxon
	 * @param description
	 * @param postOperationEnabled
	 * @return
	 */
	protected AbstractPostOperation operationCreationInstance(String label, ExecutionEvent event, DescriptionBase<?> description, IPostOperationEnabled postOperationEnabled) {
		Feature feature = (Feature) ((Event) event.getTrigger()).data;
		return new CreateDescriptionElementOperation(label, EditorUtil.getUndoContext(), description, feature, postOperationEnabled);
	}

}
