/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.concept.graph;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.zest.core.viewers.GraphViewer;
import org.eclipse.zest.core.widgets.ZestStyles;
import org.eclipse.zest.layouts.LayoutAlgorithm;
import org.eclipse.zest.layouts.LayoutStyles;
import org.eclipse.zest.layouts.algorithms.TreeLayoutAlgorithm;

import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;

/**
 * @author andreas
 *
 */
public class ConceptGraphView extends AbstractCdmEditorViewPart {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.concept.graph"; //$NON-NLS-1$

	private GraphViewer viewer;
	private LayoutAlgorithm layoutAlgoritm;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	protected void selectionChanged_internal(IWorkbenchPart part, ISelection selection) {

//		if(part instanceof ConceptGraphView){
//			// ConceptGraphView is maximized
//			return;
//		}

		if(EditorUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}

		if(part instanceof BulkEditor){
			showEmptyPage();
			return;
		}


		if(part instanceof MultiPageTaxonEditor){
			if(! part.equals(this.part)){
				IEditorInput input = ((IEditorPart) part).getEditorInput();
				showViewer(part, new StructuredSelection(input));
			}
			showViewer();
		}

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#onComplete()
	 */
	@Override
	public boolean onComplete() {
		// TODO IGNORED
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.view.AbstractCdmViewPart#getViewer()
	 */
	@Override
	public Viewer getViewer() {
		return viewer;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.view.AbstractCdmViewPart#createViewer(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createViewer(Composite parent) {

		viewer = new GraphViewer(parent, SWT.NONE);
		getSite().setSelectionProvider(viewer);

		viewer.setContentProvider(new ConceptGraphContentProvider());
		viewer.setLabelProvider(new ConceptGraphLabelProvider());

		viewer.setConnectionStyle(ZestStyles.CONNECTIONS_DIRECTED);

		viewer.setLayoutAlgorithm(getLayoutAlgoritm(), false);

//		createMenu();
//
//		createToolbar();


	}

	private LayoutAlgorithm getLayoutAlgoritm() {

		if (layoutAlgoritm == null) {
			// layoutAlgoritm = new CompositeLayoutAlgorithm(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING,
			// new LayoutAlgorithm[] {
			// new TreeLayoutAlgorithm(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING),
			// new HorizontalShift(
			// LayoutStyles.NO_LAYOUT_NODE_RESIZING) });

			layoutAlgoritm = new TreeLayoutAlgorithm(
					LayoutStyles.NO_LAYOUT_NODE_RESIZING);
			layoutAlgoritm.setEntityAspectRatio(0.5);
		}
		return layoutAlgoritm;
	}

	@Override
	protected String getViewName() {
		return Messages.ConceptGraphView_VIEWER_NAME;
	}


}
