package eu.etaxonomy.taxeditor.editor.view.derivate.contextMenu;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Context menu for the SingleReads in the derivate hierarchy.
 *
 */
public class CreateFieldUnitContextMenu extends CompoundContributionItem {

    private enum CommandType {
        COPY_TO_CLIPBOARD, REMOVE_FROM_SEQUENCE, ADD_TO_SEQUENCE
    }

    @Override
    protected IContributionItem[] getContributionItems() {
        IContributionItem[] contributionItems = new IContributionItem[] {
                new ContributionItem() {
                    @Override
                    public void fill(Menu menu, int index) {
                        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                        IWorkbenchPart activePart = window.getActivePage().getActivePart();
                        if(activePart instanceof DerivateView){
                            final DerivateView derivateView = (DerivateView) activePart;
                            if(derivateView.isListenToSelectionChange()){
                                Object selectionInput = derivateView.getSelectionInput();
                                if(selectionInput instanceof CdmBase && ((CdmBase) selectionInput).isInstanceOf(Taxon.class)){
                                    final Taxon taxon = HibernateProxyHelper.deproxy(CdmStore.getService(ITaxonService.class).load(((CdmBase)selectionInput).getUuid()), Taxon.class);
                                    MenuItem item = new MenuItem(menu, SWT.NONE);
                                    item.setText(String.format(Messages.CreateFieldUnitContextMenu_CREATE_FIELD_UNIT_FOR, taxon.getName()));
                                    item.addSelectionListener(new SelectionAdapter() {
                                        @Override
                                        public void widgetSelected(SelectionEvent e) {
                                            if(derivateView.isDirty()){
                                                MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                                                return;
                                            }

                                            FieldUnit fieldUnit = FieldUnit.NewInstance();
                                            fieldUnit = CdmBase.deproxy(CdmStore.getService(IOccurrenceService.class).save(fieldUnit), FieldUnit.class);
                                            IndividualsAssociation association = IndividualsAssociation.NewInstance(fieldUnit);
                                            TaxonDescription description;
                                            if(!taxon.getDescriptions().isEmpty()){
                                                description = taxon.getDescriptions().iterator().next();
                                            }
                                            else{
                                                description = TaxonDescription.NewInstance(taxon);
                                            }
                                            description.addElement(association);
                                            CdmStore.getService(ITaxonService.class).merge(taxon);
                                            

                                            derivateView.getConversationHolder().commit();
                                            derivateView.addFieldUnit(fieldUnit);
                                            derivateView.refreshTree(fieldUnit);
                                        }
                                    });
                                }
                            }
                            else{
                                MenuItem item = new MenuItem(menu, SWT.NONE);
                                item.setText(Messages.CreateFieldUnitContextMenu_CREATE_FIELD_UNIT);
                                item.addSelectionListener(new SelectionAdapter() {
                                    @Override
                                    public void widgetSelected(SelectionEvent e) {
                                        if(derivateView.isDirty()){
                                            MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                                            return;
                                        }
                                        FieldUnit fieldUnit = FieldUnit.NewInstance();
                                        fieldUnit = CdmBase.deproxy(CdmStore.getService(IOccurrenceService.class).save(fieldUnit), FieldUnit.class);
                                        derivateView.getConversationHolder().commit();
                                        derivateView.addFieldUnit(fieldUnit);
                                        derivateView.updateRootEntities();
                                        derivateView.refreshTree(fieldUnit);
                                    }
                                });
                            }
                        }
                    }
                }
        };
        return contributionItems;
    }
}

