package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import java.util.Collections;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.handler.OpenDerivateViewHandler;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.handler.defaultHandler.DefaultOpenHandlerBase;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class OpenDerivativeEditorForDescriptionElement extends DefaultOpenHandlerBase<IndividualsAssociation> {

    @Override
    protected IndividualsAssociation getEntity(UUID uuid) {
        DescriptionElementBase descriptionElement = CdmStore.getService(IDescriptionService.class).loadDescriptionElement(uuid, null);
        if(descriptionElement!=null && descriptionElement.isInstanceOf(IndividualsAssociation.class)){
            return (IndividualsAssociation) descriptionElement;
        }
        return null;
    }

    @Override
    protected void open(ExecutionEvent event, IndividualsAssociation entity) {
    	if(entity==null || entity.getAssociatedSpecimenOrObservation()==null){
    		MessagingUtils.messageDialog(Messages.OpenDerivativeEditorForDescriptionElement_NO_SPECIMENS, this, Messages.OpenDerivativeEditorForDescriptionElement_NO_SPECIMENS_MESSAGE);
    		return;
    	}
        SpecimenOrObservationBase associatedSpecimenOrObservation = entity.getAssociatedSpecimenOrObservation();
        if(associatedSpecimenOrObservation!=null){
            UUID specimenUuid = associatedSpecimenOrObservation.getUuid();
            try {
                EditorUtil.open(new DerivateViewEditorInput(Collections.singletonList(specimenUuid)));
            } catch (PartInitException e) {
                MessagingUtils.error(OpenDerivateViewHandler.class, OpenDerivativeEditorForTaxonNode.OPEN_DERIVATIVE_EDITOR_FOR_TAXON_NODE_COULD_NOT_OPEN, e);
            }
        }

    }

}
