package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.config.FindOccurrencesConfigurator;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.handler.OpenDerivateViewHandler;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateViewEditorInput;
import eu.etaxonomy.taxeditor.handler.defaultHandler.DefaultOpenHandlerBase;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class OpenDerivativeEditorForTaxonNode extends DefaultOpenHandlerBase<TaxonNode> {

    protected static final String OPEN_DERIVATIVE_EDITOR_FOR_TAXON_NODE_COULD_NOT_OPEN = Messages.OpenDerivativeEditorForTaxonNode_COULD_NOT_OPEN;

    @Override
    protected TaxonNode getEntity(UUID uuid) {
        return CdmStore.getService(ITaxonNodeService.class).load(uuid);
    }

    @Override
    protected void open(ExecutionEvent event, TaxonNode entity) {
        FindOccurrencesConfigurator config = new FindOccurrencesConfigurator();
        UUID taxonUuid = entity.getTaxon().getUuid();
        config.setAssociatedTaxonUuid(taxonUuid);
        List<SpecimenOrObservationBase> specimens = CdmStore.getService(IOccurrenceService.class).findByTitle(config).getRecords();
        Collection<UUID> specimenUuids = new ArrayList<UUID>();
        for (SpecimenOrObservationBase specimenOrObservationBase : specimens) {
            specimenUuids.add(specimenOrObservationBase.getUuid());
        }
        try {
            EditorUtil.open(new DerivateViewEditorInput(specimenUuids, taxonUuid));
        } catch (PartInitException e) {
            MessagingUtils.error(OpenDerivateViewHandler.class, OPEN_DERIVATIVE_EDITOR_FOR_TAXON_NODE_COULD_NOT_OPEN, e);
        }

    }

}
