package eu.etaxonomy.taxeditor.editor.view.derivate.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;

import eu.etaxonomy.cdm.api.service.molecular.ISequenceService;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class SingleReadHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        IEditorPart activeEditor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
        ISelection clipBoardSelection = LocalSelectionTransfer.getTransfer().getSelection();
        final ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getSelection();
        TreeNode clipBoardTreeNode = EditorUtil.getTreeNodeOfSelection(clipBoardSelection);
        TreeNode selectedTreeNode = EditorUtil.getTreeNodeOfSelection(selection);
        if(activeEditor instanceof DerivateView) {
            DerivateView derivateView = (DerivateView)activeEditor;
            if(event.getCommand().getId().equals("eu.etaxonomy.taxeditor.editor.derivative.copySingleReadToClipBoard")){ //$NON-NLS-1$
                LocalSelectionTransfer.getTransfer().setSelection(selection);
            }
            else if(event.getCommand().getId().equals("eu.etaxonomy.taxeditor.editor.derivative.reuseSingleRead")){ //$NON-NLS-1$
                if(activeEditor.isDirty()){
                    MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                    return null;
                }
                if(clipBoardTreeNode!=null && clipBoardTreeNode.getValue() instanceof SingleRead
                        && selectedTreeNode!=null && selectedTreeNode.getValue() instanceof Sequence){
                    SingleRead singleRead = (SingleRead)clipBoardTreeNode.getValue();
                    Sequence sequence = (Sequence)selectedTreeNode.getValue();
                    sequence.addSingleRead(singleRead);
                    CdmStore.getService(ISequenceService.class).merge(sequence);
                   
                    if(activeEditor instanceof DerivateView) {
                        derivateView.getConversationHolder().commit();
                        derivateView.refreshTree();
                        LocalSelectionTransfer.getTransfer().setSelection(null);
                    }
                }
            }
            else if(event.getCommand().getId().equals("eu.etaxonomy.taxeditor.editor.derivative.removeSingleReadFromSequence")){ //$NON-NLS-1$
                if(activeEditor.isDirty()){
                    MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                    return null;
                }
                if(selectedTreeNode!=null && selectedTreeNode.getParent()!=null &&
                        selectedTreeNode.getValue() instanceof SingleRead && selectedTreeNode.getParent().getValue() instanceof Sequence) {
                    Sequence sequence = (Sequence) selectedTreeNode.getParent().getValue();
                    sequence.removeSingleRead((SingleRead) selectedTreeNode.getValue());
                    CdmStore.getService(ISequenceService.class).merge(sequence);
                    
                    if(activeEditor instanceof DerivateView) {
                        derivateView.getConversationHolder().commit();
                        derivateView.refreshTree();
                    }
                }
            }
            derivateView.updateLabelCache();
            derivateView.refreshTree();
        }
        return null;
    }

}
