/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;

/**
 * Property tester used by the description tree menu.
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @version $Id: $
 */
public class DescriptionsMenuPropertyTester extends PropertyTester {

	private static final String MEDIA = "isMedia"; //$NON-NLS-1$
	private static final String FEATURE_NODE_CONTAINER = "isFeatureNodeContainer"; //$NON-NLS-1$
	private static final String DESCRIPTION = "isDescription"; //$NON-NLS-1$
	private static final String INDIVIDUALS_ASSOCIATION = "isIndividualsAssociation"; //$NON-NLS-1$
	private static final String DESCRIPTION_ELEMENT = "isDescriptionElement"; //$NON-NLS-1$
	private static final String DELETABLE = "isDeletable"; //$NON-NLS-1$
	private static final String IMAGE_GALLERY = "isImageGallery"; //$NON-NLS-1$
	private static final String TAXON_EDITOR = "isTaxonEditor"; //$NON-NLS-1$
	private static final String BULK_EDITOR = "isBulkEditor"; //$NON-NLS-1$
	private static final String DERIVATE_EDITOR = "isDerivateEditor"; //$NON-NLS-1$

	/* (non-Javadoc)
	 * @see org.eclipse.core.expressions.IPropertyTester#test(java.lang.Object, java.lang.String, java.lang.Object[], java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {

	    if(TAXON_EDITOR.equals(property)){
	        return isTaxonEditor();
	    }
	    else if(BULK_EDITOR.equals(property)){
	        return isBulkEditor();
	    }
	    else if(DERIVATE_EDITOR.equals(property)){
	        return isDerivateEditor();
	    }

	    Object[] selectedElements = ((IStructuredSelection) receiver).toArray();

	    if(selectedElements.length == 0){
			// nothing selected so all tests should fail
			return false;
		}

		if(MEDIA.equals(property)){
			return isMedia(selectedElements);
		}
		else if(FEATURE_NODE_CONTAINER.equals(property)){
			return isFeatureNodeContainer(selectedElements);
		}
		else if(DESCRIPTION.equals(property)){
			return isDescription(selectedElements);
		}
		else if(INDIVIDUALS_ASSOCIATION.equals(property)){
		    return isIndividualsAssociation(selectedElements);
		}
		else if(DESCRIPTION_ELEMENT.equals(property)){
			return isDescriptionElement(selectedElements);
		}
		else if(DELETABLE.equals(property)){
			return isDeletable(selectedElements);
		}
		else if(IMAGE_GALLERY.equals(property)){
			return isImageGallery(selectedElements);
		}
		else{
			return false;
		}
	}

	private boolean isImageGallery(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionBase) || !((DescriptionBase<?>) object).isImageGallery()){
				return false;
			}
		}
		return true;
	}

	private boolean isFeatureNodeContainer(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof FeatureNodeContainer)){
				return false;
			}
		}
		return true;
	}

	private boolean isDeletable(Object[] selectedElements) {
		boolean result = false;

		for (Object object : selectedElements) {

			if((object instanceof DescriptionBase)
				|| (object instanceof DescriptionElementBase)
				|| (object instanceof Media)
				|| (object instanceof FeatureNodeContainer)){
				result = true;
			}else{
				return false;
			}

		}
		return result;
	}

	private boolean isDescriptionElement(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionElementBase)){
				return false;
			}
		}
		return true;
	}

	private boolean isDescription(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionBase)){
				return false;
			}
		}
		return true;
	}

	private boolean isIndividualsAssociation(Object[] selectedElements) {
	    for (Object object : selectedElements){
	        if(!(object instanceof IndividualsAssociation)){
	            return false;
	        }
	    }
	    return true;
	}

	private boolean isMedia(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof Media)){
				return false;
			}
		}
		return true;
	}

	private boolean isTaxonEditor() {
	    if(AbstractUtility.getActiveEditor() instanceof MultiPageTaxonEditor){
	        return true;
	    }
	    return false;
	}

	private boolean isBulkEditor() {
	    if(AbstractUtility.getActiveEditor() instanceof BulkEditor){
	        return true;
	    }
	    return false;
	}

	private boolean isDerivateEditor() {
	    if(AbstractUtility.getActiveEditor() instanceof DerivateView){
	        return true;
	    }
	    return false;
	}
}
