/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.utility.ImagesUtility;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * <p>RemoveImageFromDescriptionElementOperation class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 05.02.2009
 * @version 1.0
 */
public class RemoveImageFromDescriptionElementOperation extends AbstractPostTaxonOperation {

	private DescriptionBase element;
	private Media media;

	/**
	 * <p>Constructor for RemoveImageFromDescriptionElementOperation.</p>
	 *
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param element a {@link eu.etaxonomy.cdm.model.description.DescriptionElementBase} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param label a {@link java.lang.String} object.
	 * @param config
	 * @param image a {@link eu.etaxonomy.cdm.model.media.ImageFile} object.
	 */
	public RemoveImageFromDescriptionElementOperation(String label,
			IUndoContext undoContext, Media media,
			DescriptionBase element, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);

		this.media = media;
		this.element = element;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		monitor.worked(20);
		ImagesUtility.removeMediaFromGallery(element, media);
		monitor.worked(40);

		return postExecute(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		ImagesUtility.addMediaToGallery(element, media);
		return postExecute(media);
	}
}
