/**
 * Copyright (C) 2011 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.view.uses;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveContentProvider;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainerTree;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.UsageTermCollection;

/**
 * The class provides the required content to the Uses View
 * @author a.theys
 * @created mar 13, 2012
 * @version 1.0
 */
public class UsesContentProvider extends DescriptiveContentProvider {

	Set<MarkerType> markerTypes = new HashSet<MarkerType>();

	/**
	 * <p>Constructor for DescriptiveContentProvider.</p>
	 *
	 * @param featureNodeContainerCache a {@link java.util.Map} object.
	 */
	public UsesContentProvider(Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache) {
		super(featureNodeContainerCache);
		this.featureNodeContainerCache = featureNodeContainerCache;
	}

	/**
	 * Get all descriptions associated with the given TaxonEditorInput
	 *
	 * @param parentElement
	 * @return
	 */
	@Override
	protected List<DescriptionBase<?>> getDescriptions(IDescribable<?> parentElement) {
	    List<DescriptionBase<?>> descriptions = new ArrayList<DescriptionBase<?>>();
	    if(parentElement instanceof Taxon){
	        Taxon taxon = (Taxon) parentElement;
	        this.markerTypes.addAll(CdmStore.getTermManager().getPreferredTerms(MarkerType.class));
	        for(DescriptionBase<?> description : taxon.getDescriptions()){
	            if(! description.isImageGallery()){
	            	Set<Marker> descriptionMarkers = description.getMarkers();
	                if(descriptionMarkers != null) {
	                    for (Marker marker: descriptionMarkers) {
	                        if(marker.getMarkerType() != null && marker.getMarkerType().getUuid().equals(UsageTermCollection.uuidUseMarkerType)) {
	                            descriptions.add(description);
	                        }
	                    }
	                }
	            }
	        }
	    }
		return descriptions;
	}

}
