/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import junit.framework.Assert;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TaxonNameFactory;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.name.operation.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.operations.AbstractTaxeditorOperationTestBase;

/**
 * @author n.hoffmann
 * @created 02.04.2009
 * @version 1.0
 */
public class ChangeHomotypicGroupOperationTest extends AbstractTaxeditorOperationTestBase {
	private static final Logger logger = Logger
			.getLogger(ChangeHomotypicGroupOperationTest.class);

	
	private static AbstractPostOperation operation;


	private static Taxon taxon;


	private static Synonym synonym;


	private static HomotypicalGroup newHomotypicalGroup;


	private static HomotypicalGroup oldHomotypicalGroup;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		
		taxon = Taxon.NewInstance(null, null);
		synonym = Synonym.NewInstance(TaxonNameFactory.NewNonViralInstance(null), null);
		
		oldHomotypicalGroup = HomotypicalGroup.NewInstance();
		oldHomotypicalGroup.addTypifiedName(synonym.getName());
		
		newHomotypicalGroup = HomotypicalGroup.NewInstance();
		
		operation = new ChangeHomotypicGroupOperation("Change Homotypical Group", null, taxon, synonym, newHomotypicalGroup, postOperation);
	}


	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.ChangeHomotypicGroupOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecute() throws ExecutionException {
		operation.execute(null, null);
		
		Assert.assertEquals(newHomotypicalGroup, synonym.getHomotypicGroup());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.ChangeHomotypicGroupOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndo() throws ExecutionException {
		operation.undo(null, null);
		
		Assert.assertEquals(oldHomotypicalGroup, synonym.getHomotypicGroup());
	}
	
	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.ChangeHomotypicGroupOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedo() throws ExecutionException {
		operation.redo(null, null);
		
		Assert.assertEquals(newHomotypicalGroup, synonym.getHomotypicGroup());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation#postExecute(ICdmBase)}.
	 */
	@Test
	public void testPostExecute() {
		// TODO there is not post operation functionality for this class
		// at the moment. Implement test when there is.
		logger.warn("No post operation functionality for this class");
	}
}
